
/*
 * Initial Program: Chris Barnes, Smart Fan Controller with PID
 * Modifications  : Jennifer Sirtl, Heating-Case for Lipo-Batteries
 *
 * The MIT License (MIT)
 * Copyright (c) 2014 Chris Barnes
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
 
#include <OneWire.h> //important for the temperature-sensor
#include <DallasTemperature.h>//important for the temperature-sensor

#include <PID_v1.h>//the mechanism of the regulator
#include <LiquidCrystal.h>//library for the Display
#include <EEPROM.h>//non vioitale memory 
 
//Definitions
#define Heater 9           // Output pin for Heater
#define ONE_WIRE_BUS 8  // Temperature Input is on Pin 8
#define click 3         //Rotary Encoder push button
#define encoder0PinA  2 //Rotary Encoder rotation switch1
#define encoder0PinB  4 //Rotary Encoder rotation switch2
#define TemperatureLimit 60.00  //this Limit is held, independent of malfunction of the regulator
#define EEATemperature //Adress EEprom for Temperature

volatile unsigned int encoder0Pos = 0;  //Encoder value for ISR
volatile unsigned int HeaterOnOff = 1; //Heatfunction-switch 0=off, 1=on 
volatile unsigned char DisplayPage = 0; //Normal Display
volatile float Energy = 0;//Energycounter
volatile int EEATadress = 0; //Address for the EEProm EEATemperature
LiquidCrystal lcd(12, 11, 13, 5,6,7);  //set up LCD
 
//Setup Temperature Sensor
OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);
DeviceAddress Thermometer; // arrays to hold device address

//Setup PID
double Setpoint, Input, Output;  //I/O for PID
double Kp=85, Ki=20, Kd=0.5;      //PID settings
PID myPID(&Input, &Output, &Setpoint, Kp, Ki, Kd, DIRECT);  //Initialize PID
int timeCounter; //On-Time 

//interface
void setup(void)
{  
  pinMode(ONE_WIRE_BUS, INPUT_PULLUP);               

  // start serial port for temperature readings
  Serial.begin(9600);
  Serial.println("Start");
  
  //Temperature Setup
  sensors.begin();     //Start Library
  
  //This line Initalises Thermometer:
  if (!sensors.getAddress(Thermometer, 0)) Serial.println("Unable to find address for Device 0"); //Very important!
  
  
  sensors.setResolution(Thermometer,12);
  Serial.print("Thermometer Resolution: ");
  Serial.print(sensors.getResolution(Thermometer), DEC); 
  Serial.println();
  
  //sensors.requestTemperatures();      // Send the command to get temperatures
  Input = sensors.getTempCByIndex(0); //Set Input to Current Temperature
  if (sensors.getAddress(Thermometer, 0)) 
  {
    HeaterOnOff=1;
  }
  else
  {
    HeaterOnOff=0; //Temperaturesensor is defect
  }
  
  //Setpoint initialisation
  Setpoint =  (double)EEPROM.read(EEATadress);
  if (Setpoint == 255)
  { 
    Setpoint = 45;                      //Inintialize desired Temperature in Deg C
    encoder0Pos=Setpoint;
  }
  else
  {
    encoder0Pos=Setpoint;    
  }
  
  //PID Setup
  myPID.SetMode(AUTOMATIC);
  myPID.SetTunings(Kp, Ki, Kd);
 
  //Setup Pins
  pinMode(Heater, OUTPUT);                   // Output for Heater-PWM 0 to 255
  pinMode(click, INPUT);                  // Click button is an input
  //pinMode(click, INPUT_PULLUP);
  pinMode(encoder0PinA, INPUT); 
  digitalWrite(encoder0PinA, HIGH);       // Turn on pullup resistor
  pinMode(encoder0PinB, INPUT); 
  digitalWrite(encoder0PinB, HIGH);       // Turn on pullup resistor
 
  //Set up Interupts
  attachInterrupt(1, clicked, RISING);    // Click button on interrupt 1 - pin 3
  attachInterrupt(0, doEncoder, CHANGE);  // Encoder pin on interrupt 0 - pin 2
 
  //interface
  timeCounter=0;
 
  //Setup LCD 16x2 and display startup message
  lcd.begin(16, 2);
  lcd.print("Heizkoffer 1.2.0");//Program and version
  lcd.setCursor(0,1);
  lcd.print("Jennifer Sirtl"); //Owner
  delay(3000);
  lcd.clear();
}
void loop()
{
   timeCounter++;
 
   //Get temperature and give it to the PID input
   sensors.requestTemperatures();
   Input=sensors.getTempCByIndex(0);
   if (!sensors.getAddress(Thermometer, 0))
   {
     HeaterOnOff=0; //Temperaturesensor faulty
   }
 
  //print out info to LCD
  lcd.setCursor(0,0);
  lcd.print("Temp:");
  lcd.print((int)Input);
  lcd.print("\337");
  lcd.print("C");
  lcd.setCursor(10,0);
  if (HeaterOnOff==1)
    {
      if (DisplayPage==0)
      {
      lcd.print("H:");
      lcd.setCursor(12,0);
      lcd.print("    ");
      lcd.setCursor(12,0);  
      lcd.print((int)(Output/2.55));    //Scaling to 100%
      lcd.print("%");
      lcd.setCursor(0,1);
      lcd.print("Wunsch:");
      lcd.print((int)Setpoint);
      lcd.print("\337");
      lcd.print("C ");
      }
      else
      {
      lcd.print((int)(Energy/637));    
      lcd.print("Ah");
      lcd.setCursor(0,1);
      lcd.print("Avg:");
      lcd.print((int)(Energy/timeCounter*100));
      lcd.print("%");
      }
    }
  else
    {
    lcd.print("--     ");
    lcd.setCursor(0,1);
    lcd.print("Heizung ausg.   ");
    }

 
  //Compute PID value
  double gap = abs(Setpoint-Input); //distance away from setpoint
  //myPID.SetTunings(Kp, Ki, Kd);
  myPID.Compute();
  Serial.print(timeCounter);
  Serial.print(";");
  Serial.print(Input);
  Serial.print(";");
  Serial.print(Output);
  Serial.print(";");
  Serial.println(Energy);
 
  if (HeaterOnOff==1)
    {
     //Write PID output to Heater if not critical
    if (Input<TemperatureLimit)
      {
      analogWrite(Heater,Output);
      Energy = Energy + (Output/255);
      }
    else
      {
      analogWrite(Heater,0);
      }  
  }
  else
    {
    analogWrite(Heater,0);
    }  
}
 
void doEncoder()
{
  //pinA and pinB are both high or both low, spinning forward, otherwise it's spinning backwards
  if (digitalRead(encoder0PinA) == digitalRead(encoder0PinB))
  { 
    if (encoder0Pos<55) 
      {encoder0Pos++;}
  }
  else
  {
    if (encoder0Pos>0) 
      {encoder0Pos--;}    
  }
  Setpoint=encoder0Pos;
  EEPROM.write(EEATadress,(unsigned char)encoder0Pos);
}

void clicked()
{
  lcd.clear();

  if (DisplayPage==1)
    {
      DisplayPage=0;
     }
  else
     {
      DisplayPage=1;
     }  
}

